#========================================================================================
# package      DE_EPAGES::BacklinkProvider::t::unit::UI_Provider
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::BacklinkProvider::t::unit::UI_Provider;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::BacklinkProvider::UI::Provider;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant URL   => 'https://teamoceanbcn-sea.intern.epages.de/MBOStuff';
use constant Token => 'ThisIsMyToken';

sub test {
    my $self = shift;

    $self->_test_RedirectToURL_WithValidURLAndToken();
    $self->_test_RedirectToURL_WithInvalidURL();
    $self->_test_RedirectToURL_WithoutToken();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_RedirectToURL_WithValidURLAndToken {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIProvider = $self->_createUIProvider();
    $MockedUIProvider->RedirectToURL( URL, Token );

    my $aParameters = GetParametersFromMockifyCall( $MockedUIProvider, '_redirect' );
    is_deeply( $aParameters, [ URL ],
        "$SubTestName - _redirect was called with the right parameters." );

    return;
}

sub _test_RedirectToURL_WithInvalidURL {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIProvider = $self->_createUIProvider();
    $MockedUIProvider->RedirectToURL();

    ok( !WasCalled( $MockedUIProvider, '_redirect' ), "$SubTestName - _redirect was not called." );

    return;
}

sub _test_RedirectToURL_WithoutToken {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIProvider = $self->_createUIProvider();
    $MockedUIProvider->RedirectToURL( URL );

    my $aParameters = GetParametersFromMockifyCall( $MockedUIProvider, '_redirect' );
    is_deeply( $aParameters, [ URL ],
        "$SubTestName - _redirect was called with the right parameters." );

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createUIProvider {
    my $self = shift;

    my $MockifyUIProvider = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::BacklinkProvider::UI::Provider');

    my $aParams = [ 'string' ];
    $MockifyUIProvider->addMockWithReturnValueAndParameterCheck( '_redirect', undef, $aParams );

    return $MockifyUIProvider->getMockObject();
}

__PACKAGE__->runScript();
