package DE_EPAGES::Content::t::Category;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Content::API::Object::Category;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_checkCategoryNesting();
    done_testing();

    return;
}

sub test_checkCategoryNesting {
    my $self = shift;

    my $Shop = $self->_createMockedShop();
    my $StartPage = $Shop->children->[0];
    my $Category1 = $StartPage->children->[0];
    my $Category2 = $Category1->children->[0];
    my $Category3 = $Category2->children->[0];
    my $Category4 = $Category3->children->[0];

    is( $StartPage->countCategoryNesting, 0, 'Category Depth of StartPage = 0' );
    is( $Category1->countCategoryNesting, 1, 'Category Depth of Level1 = 1' );
    is( $Category4->countCategoryNesting, 4, 'Category Depth of Level4 = 4' );

    is( $StartPage->countChildrenDepth, 4, 'Children Depth of StartPage = 4' );
    is( $Category1->countChildrenDepth, 3, 'Children Depth of Level1 = 3' );
    is( $Category3->countChildrenDepth, 1, 'Children Depth of Level3 = 1' );
    is( $Category4->countChildrenDepth, 0, 'Children Depth of Level4 = 0' );

    return;
}

sub _createMockedShop {
    my $self = shift;

    my $Category4 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'Level4',
            'instanceOf' => 'Category',
            'Children' => []
        }
    )->getMockObject();

    my $Category3 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'Level3',
            'instanceOf' => 'Category',
            'Children' => [ $Category4 ]
        }
    )->getMockObject();

    my $Category2 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'Level2',
            'instanceOf' => 'Category',
            'Children' => [ $Category3 ]
        }
    )->getMockObject();


    my $Category1 = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Content::API::Object::Category', {
            'Alias' => 'Level1',
            'instanceOf' => 'Category',
            'Children' => [ $Category2 ]
        }
    )->getMockObject();


    my $StartPage = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Content::API::Object::StartPage', {
            'Alias' => 'StartPage',
            'instanceOf' => 'Category',
            'Children' => [ $Category1 ]
        }
    )->getMockObject();


    my $Shop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', {
            'Alias'             => 'TestShop',
            'IsUnityRegistered' => 1,
            'instanceOf' => 'Shop',
            'Children' => [ $StartPage ]
        }
    )->getMockObject();


    $StartPage->set({ 'Parent' => $Shop });
    $Category1->set({ 'Parent' => $StartPage });
    $Category2->set({ 'Parent' => $Category1 });
    $Category3->set({ 'Parent' => $Category2 });
    $Category4->set({ 'Parent' => $Category3 });

    return $Shop;
}

__PACKAGE__->runScript();
1;