#========================================================================================
# package      DE_EPAGES::Content::t::RestApiIntegrationTest
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for RestApi resources.
#               Checks if requests are matched by the correct resources.
#========================================================================================
package DE_EPAGES::Content::t::RestApiIntegrationTest;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::WebService::API::ResourceRegistry;
use HTTP::Request;
use Test::Exception;
use Test::More;

use strict;



sub shopAlias {
    return 'TestShop';
}



sub importXMLPath {
    return 'import.xml';
}



sub test {
    my $self = shift;

    $self->test_get_legalInformationSummary();

    $self->test_get_imprint();
    $self->test_put_imprint();

    $self->test_get_customerInformation();
    $self->test_put_customerInformation();

    $self->test_get_privacyPolicy();
    $self->test_put_privacyPolicy();

    $self->test_get_shippingInformation();
    $self->test_put_shippingInformation();

    $self->test_get_termsAndConditions();
    $self->test_put_termsAndConditions();

    done_testing();
}



sub test_get_legalInformationSummary {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::LegalInformationSummaryResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_get_imprint {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/imprint/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::ImprintResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_imprint {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('PUT', '/api/legal/imprint/?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::ImprintResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], {}, "$SubTestName - test if the content was extracted correctly");
    is($aParams->[2], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_get_customerInformation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/customerinformation/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::CustomerInformationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_customerInformation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('PUT', '/api/legal/customerinformation/?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::CustomerInformationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], {}, "$SubTestName - test if the content was extracted correctly");
    is($aParams->[2], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_get_privacyPolicy {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/privacypolicy/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::PrivacyPolicyResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_privacyPolicy {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('PUT', '/api/legal/privacypolicy/?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::PrivacyPolicyResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], {}, "$SubTestName - test if the content was extracted correctly");
    is($aParams->[2], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_get_shippingInformation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/shippinginformation/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::ShippingInformationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_shippingInformation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('PUT', '/api/legal/shippinginformation/?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::ShippingInformationResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], {}, "$SubTestName - test if the content was extracted correctly");
    is($aParams->[2], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}


sub test_get_termsAndConditions {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/legal/termsandconditions/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::TermsAndConditionsResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub test_put_termsAndConditions {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('PUT', '/api/legal/termsandconditions/?locale=de_DE', 'application/vnd.epages.v1+json', '{}');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Content::API::Resource::TermsAndConditionsResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'put', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is_deeply($aParams->[1], {}, "$SubTestName - test if the content was extracted correctly");
    is($aParams->[2], 'de_DE', "$SubTestName - test if the locale was extracted correctly");
}



sub _createRequest {
    my $self = shift;
    my ($Method, $Route, $Accept, $Content) = @_;

    my $URL = 'http://www.example.com'.$Route;
    my $Request = HTTP::Request->new($Method => $URL);

    if(defined $Accept) {
        $Request->header( 'Accept' => $Accept );
    }
    if(defined $Content) {
        warn "set content: $Content \n";
        $Request->content( $Content );
    }

    return $Request;
}



__PACKAGE__->runScript();

1;