#========================================================================================
# package      DE_EPAGES::Order::t::unit::LineItemPriceCalculator
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::LineItemPriceCalculator
#========================================================================================
package DE_EPAGES::Order::t::unit::LineItemPriceCalculator;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::LineItemPriceCalculator;
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_GROSS TAX_MODEL_NET );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;
use Test::Exception;

use strict;



sub test {
    my $self = shift;

    $self->test_calculate_GrossShop_IncludeTaxFlag_False();
    $self->test_calculate_GrossShop_IncludeTaxFlag_True();

    $self->test_calculate_GrossShop_IncludeTaxFlag_False_Digital();
    $self->test_calculate_GrossShop_IncludeTaxFlag_True_Digital();

    $self->test_calculate_NetShop_IncludeTaxFlag_False();
    $self->test_calculate_NetShop_IncludeTaxFlag_True();

    $self->test_calculate_NetShop_IncludeTaxFlag_False_Digital();
    $self->test_calculate_NetShop_IncludeTaxFlag_True_Digital();

    done_testing();
}



sub test_calculate_GrossShop_IncludeTaxFlag_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_GrossShop();
    my $LineItemProduct = $Mockery->get('LineItemProductGrossShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'false';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '27.4916666666667',
          'BasePrice' => '27.4916666666667'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right");
}



sub test_calculate_GrossShop_IncludeTaxFlag_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_GrossShop();
    my $LineItemProduct = $Mockery->get('LineItemProductGrossShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'true';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_GROSS,
          'LineItemPrice' => '32.99',
          'BasePrice' => '32.99'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub test_calculate_GrossShop_IncludeTaxFlag_False_Digital {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseDigitalTax = 1;
    my $Mockery = $self->_createObjects_GrossShop($UseDigitalTax);
    my $LineItemProduct = $Mockery->get('LineItemProductGrossShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'false';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '23.5642857142857',
          'BasePrice' => '23.5642857142857'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right");
}



sub test_calculate_GrossShop_IncludeTaxFlag_True_Digital {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseDigitalTax = 1;
    my $Mockery = $self->_createObjects_GrossShop($UseDigitalTax);
    my $LineItemProduct = $Mockery->get('LineItemProductGrossShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'true';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_GROSS,
          'LineItemPrice' => '32.99',
          'BasePrice' => '32.99'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub test_calculate_NetShop_IncludeTaxFlag_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NetShop();
    my $LineItemProduct = $Mockery->get('LineItemProductNetShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'false';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '27.49',
          'BasePrice' => '27.49'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub test_calculate_NetShop_IncludeTaxFlag_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NetShop();
    my $LineItemProduct = $Mockery->get('LineItemProductNetShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'true';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '27.49',
          'BasePrice' => '27.49'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub test_calculate_NetShop_IncludeTaxFlag_False_Digital {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseDigitalTax = 1;
    my $Mockery = $self->_createObjects_NetShop($UseDigitalTax);
    my $LineItemProduct = $Mockery->get('LineItemProductNetShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'false';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '27.49',
          'BasePrice' => '27.49'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub test_calculate_NetShop_IncludeTaxFlag_True_Digital {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UseDigitalTax = 1;
    my $Mockery = $self->_createObjects_NetShop($UseDigitalTax);
    my $LineItemProduct = $Mockery->get('LineItemProductNetShop');

    my $LineItemPriceCalculator = $Mockery->get('SUT');
    my $CurrencyID = 'EUR';
    my $IncludeTaxFlag = 'true';
    my $hLineItemContainerData = $LineItemPriceCalculator->calculate($LineItemProduct, $CurrencyID, $IncludeTaxFlag);

    my $hExpectedLineItemContainerData = {
          'TaxModelLineItem' => TAX_MODEL_NET,
          'LineItemPrice' => '27.49',
          'BasePrice' => '27.49'
    };
    is_deeply($hLineItemContainerData, $hExpectedLineItemContainerData, "$SubTestName - test if price was calculate right.");
}



sub _createObjects_GrossShop {
    my $self = shift;
    my ($UseDigitalTax) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainerGrossShop', ['LineItemContainer', 'LineItemProductGrossShop'], sub {
        my ($LineItemContainer, $LineItemProduct) = @_;
        $LineItemContainer->{'Attributes'}->{'TaxModel'} = TAX_MODEL_GROSS;
        $LineItemContainer->{'Attributes'}->{'LineItems'} = [ $LineItemProduct ];
        $LineItemProduct->{'Attributes'}->{'LineItemContainer'} = $LineItemContainer;
        return $LineItemContainer;
    });

    $Mockery->register('LineItemProductGrossShop', ['LineItemProduct'], sub {
        my ($LineItemProduct ) = @_;
        $LineItemProduct->{'Attributes'}->{'LineItemPrice'} = 32.99;
        $LineItemProduct->{'Attributes'}->{'BasePrice'} = 32.99;
        return $LineItemProduct;
    });

    $Mockery->extend($self->_createCommonMocks($UseDigitalTax));
    return $Mockery->assemble();
}



sub _createObjects_NetShop {
    my $self = shift;
    my ($UseDigitalTax) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainerNetShop', ['LineItemContainer', 'LineItemProductNetShop'], sub {
        my ($LineItemContainer, $LineItemProduct) = @_;
        $LineItemContainer->{'Attributes'}->{'TaxModel'} = TAX_MODEL_NET;
        $LineItemContainer->{'Attributes'}->{'LineItems'} = [ $LineItemProduct ];
        $LineItemProduct->{'Attributes'}->{'LineItemContainer'} = $LineItemContainer;
        return $LineItemContainer;
    });

    $Mockery->register('LineItemProductNetShop', ['LineItemProduct'], sub {
        my ($LineItemProduct ) = @_;
        $LineItemProduct->{'Attributes'}->{'LineItemPrice'} = 27.49;
        $LineItemProduct->{'Attributes'}->{'BasePrice'} = 27.49;
        return $LineItemProduct;
    });

    $Mockery->extend($self->_createCommonMocks($UseDigitalTax));
    return $Mockery->assemble();
}



sub _createCommonMocks {
    my $self = shift;
    my ($UseDigitalTax) = @_;

    my $TaxClassName = $UseDigitalTax ? 'TaxClassDigital' : 'TaxClass';
    my $TaxAreaName = $UseDigitalTax ? 'TaxAreaDigital' : 'TaxArea';
    my $TaxMatrixName = $UseDigitalTax ? 'TaxMatrixDigital' : 'TaxMatrix';

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', [], sub {
        my $LineItemPriceCalculator = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemPriceCalculator', []);
        $LineItemPriceCalculator->addMockWithReturnValueAndParameterCheck('_getPrecision', 2, ['string']);
        return $LineItemPriceCalculator->getMockObject();
    });

    $Mockery->register('LineItemContainer', [$TaxAreaName, $TaxMatrixName], sub {
        my ($TaxArea, $TaxMatrix) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'CurrencyID' => 'EUR',
            'GrandTotal' => '123.45',
            'TotalBeforeTax' => '56',
            'TaxArea' => $TaxArea,
            'TaxAreaDigital' => $UseDigitalTax ? $TaxArea : undef,
            'TaxMatrix' => $TaxMatrix,
            'TaxMatrixDigital' => $UseDigitalTax ? $TaxMatrix : undef,
            'TotalTax' => '23',
            'LineItemsSubTotal' => '55.2',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemProduct', ['Product1', $TaxClassName], sub {
        my ($Product1, $TaxClass) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', {
            'Quantity' => 2,
            'GUID' => '123-35-56562-324352',
            'SKU' => 'abc_12345346',
            'Name' => 'Product 1',
            'LineItemContainer' => undef,
            'Product' => $Product1,
            'TaxArea' => '',
            'TaxClass' => $TaxClass,
            'TaxClassDigital' => $UseDigitalTax ? $TaxClass : undef,
            'TaxRate' => 0.19,
        });
        $Mockify->mock('instanceOf', sub {return 1;});
        return $Mockify->getMockObject();
    });

    $Mockery->register('Product1', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'GUID' => '245345-32534-324234',
            'MainCharacteristics' => 'test product',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'Name' => 'TaxClassName',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxArea', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxArea', {
            'name' => 'EU country',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxMatrix', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Tax::API::Object::TaxMatrix');
        $Mockify->addMock('gross2net', sub {
            my $MockedSelf = shift;
            my ($CurrencyID, $Price, $hParam) = @_;
            return $Price / 1.2; #this is only for test proposals, as we mock the gross2net we just want to get a different price
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClassDigital', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClassDigital', {
            'Name' => 'TaxClassName',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxAreaDigital', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxAreaDigital', {
            'name' => 'EU country',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxMatrixDigital', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Tax::API::Object::TaxMatrixDigital');
        $Mockify->addMock('gross2net', sub {
            my $MockedSelf = shift;
            my ($CurrencyID, $Price, $hParam) = @_;
            return $Price / 1.4; #this is only for test proposals, as we mock the gross2net we just want to get a different price
        });
        return $Mockify->getMockObject();
    });

    return $Mockery;
}

__PACKAGE__->runScript();