#========================================================================================
# package      DE_EPAGES::Order::t::unit::LineItemContainerData
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::LineItemContainerData
#========================================================================================
package DE_EPAGES::Order::t::unit::LineItemsData;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::LineItemsData;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::Mockery;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->test_getLineItemsData_NormalProduct();
    $self->test_getLineItemsData_DownloadableProduct();

    done_testing();
}



sub test_getLineItemsData_NormalProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_NormalProduct();
    my $LineItemsData = $Mockery->get('SUT');

    my $LineItemContainer = $Mockery->get('LineItemContainer');
    my $Formatter = $Mockery->get('Formatter');
    my $LanguageID = 'en_GB';
    my $IncludeTaxInLineItem = 1;

    my $hLineItemsData = $LineItemsData->getLineItems($LineItemContainer, $Formatter, $LanguageID, $IncludeTaxInLineItem);
    my $hExpectedLineItemsData = $self->_createExpectedLineItemsData();
    is_deeply($hLineItemsData, $hExpectedLineItemsData, "$SubTestName - test if expected LineItemContainer data was returned.");

    my $aCalculateCall = GetParametersFromMockifyCall($Mockery->get('LineItemPriceCalculator'), 'calculate');
    my $IncludeTaxInLineItem_fromCall = $aCalculateCall->[2];
    is($IncludeTaxInLineItem, $IncludeTaxInLineItem_fromCall, "$SubTestName - Check if the IncludeTax flag was given to the PriceCalculator class");
}

sub test_getLineItemsData_DownloadableProduct {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Mockery = $self->_createObjects_WithDownloadable();
    my $LineItemsData = $Mockery->get('SUT');

    my $LineItemContainer = $Mockery->get('LineItemContainer');
    my $Formatter = $Mockery->get('Formatter');
    my $LanguageID = 'en_GB';
    my $IncludeTaxInLineItem = 1;

    my $hLineItemsData = $LineItemsData->getLineItems($LineItemContainer, $Formatter, $LanguageID, $IncludeTaxInLineItem);
    my $hExpectedLineItemsData = $self->_createExpectedLineItemsData_DownloadableProduct();
    is_deeply($hLineItemsData, $hExpectedLineItemsData, "$SubTestName - test if expected LineItemContainer data was returned.");
}

sub _createObjects_NormalProduct {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['LineItemProduct1', 'Shop' ], sub {
        my ($LineItemProduct1, $Shop) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'TaxModel' => 'GROSS',
            'CurrencyID' => 'EUR',
            'LineItems' => [
                $LineItemProduct1
            ],
            'Site' => $Shop,
        });
    });

    $Mockery->extend($self->_createCommonMocks());

    return $Mockery->assemble();
}


sub _createObjects_WithDownloadable {
    my $self = shift;

    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('LineItemContainer', ['LineItemProductDownloadable', 'Shop' ], sub {
        my ($LineItemProductDownloadable, $Shop) = @_;
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', {
            'TaxModel' => 'GROSS',
            'CurrencyID' => 'EUR',
            'LineItems' => [
                $LineItemProductDownloadable
            ],
            'Site' => $Shop,
        });
    });

    $Mockery->extend($self->_createCommonMocks());

    return $Mockery->assemble();
}

sub _createCommonMocks {
    my $self = shift;
    my $Mockery = DE_EPAGES::Test::Mock::Mockery->new();

    $Mockery->register('SUT', ['Context', 'LineItemPriceCalculator'], sub {
        my ($Context, $LineItemPriceCalculator) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemsData', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('_createShopRequestContext', $Context, [{'object'=> 'DE_EPAGES::Shop::API::Object::Shop'}]);
        $Mockify->addMockWithReturnValue('_createPriceCalculator', $LineItemPriceCalculator, []);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Shop', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Object::Shop', []);
        $Mockify->addMockWithReturnValueAndParameterCheck('siteLocale', 'en_GB', [{'string' => 'en_GB'}]);
        return $Mockify->getMockObject();
    });

    $Mockery->register('Formatter', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::Formatter', [$Shop, 'en_GB']);
        $Mockify->addMock('quantity', sub {
            my $MockedSelf = shift;
            my ($Quantity, $Unit) = @_;
            return $Quantity;
        });
        $Mockify->addMock('price', sub {
            my $MockedSelf = shift;
            my ($Price, $Currency, $TaxModel) = @_;
            return $Price;
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemProduct1', ['Product1', 'OrderUnit', 'TaxClass'], sub {
        my ($Product1, $OrderUnit, $TaxClass) = @_;
        my $LineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', {
            'BasePrice' => 32.99,
            'GUID' => '123-35-56562-324352',
            'LineItemPrice' => 32.99,
            'Name' => 'LineItemProduct',
            'OrderUnit' => $OrderUnit,
            'Quantity' => 2,
            'Product' => $Product1,
            'SKU' => 'abc_12345346',
            'TaxClass' => $TaxClass,
            'TaxClassDigital' => undef,
            'TaxRate' => 0.19,
            'Weight' => 4.89,
            'WeightQuantity' => 1,
            'WeightUnit' => 'kg',
        });
        $LineItemProduct->mock('instanceOf', sub {return 1;});
        return $LineItemProduct;
    });

    $Mockery->register('LineItemProductDownloadable', ['Product1', 'OrderUnit', 'TaxClass'], sub {
        my ($Product1, $OrderUnit, $TaxClass) = @_;
        my $LineItemProduct = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::LineItemProduct', {
            'BasePrice' => 32.99,
            'GUID' => '123-35-56562-324352',
            'LineItemPrice' => 32.99,
            'Name' => 'LineItemProductDownloadable',
            'OrderUnit' => $OrderUnit,
            'Quantity' => 2,
            'Product' => $Product1,
            'SKU' => 'abc_12345346',
            'TaxClass' => $TaxClass,
            'TaxClassDigital' => undef,
            'TaxRate' => 0.19,
            'Weight' => undef,
            'WeightQuantity' => 1,
            'WeightUnit' => undef,
        });
        $LineItemProduct->mock('instanceOf', sub {return 1;});
        return $LineItemProduct;
    });


    $Mockery->register('Product1', [], sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', {
            'GUID' => '245345-32534-324234',
            'ImageHotDeal' => 'hotdeal',
            'ImageLarge' => 'large',
            'Name' => 'Product 1',
            'ImageSmall' => 'small',
            'ImageMedium' => 'medium',
            'ImageThumbnail' => 'thumbnail',
            'WebPath' => '/web/path',
            'MainCharacteristics' => 'test product',
        });
    });

    $Mockery->register('OrderUnit', [], sub {
        return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::UnitOfMeasurement', {});
    });

    $Mockery->register('Context', ['Shop'], sub {
        my ($Shop) = @_;
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shop::API::Resource::ShopRequestContext', [$Shop]);
        $Mockify->addMockWithReturnValue('apiBaseUrl', 'http://baseurl.test.org');
        $Mockify->addMockWithReturnValue('imageBaseUrl', 'http://image.baseurl.test.org');
        return $Mockify->getMockObject();
    });

    $Mockery->register('TaxClass', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Tax::API::Object::TaxClass', {
            'GUID' => '123-456-7890',
            'Name' => 'TaxClassName',
        });
        return $Mockify->getMockObject();
    });

    $Mockery->register('LineItemPriceCalculator', [], sub {
        my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::LineItemPriceCalculator', []);
        my $hLineItemPrice = {
          'TaxModelLineItem' => 1,
          'LineItemPrice' => '32.99',
          'BasePrice' => '32.99'
        };
        $Mockify->addMockWithReturnValueAndParameterCheck('calculate', $hLineItemPrice, [{'object' => 'DE_EPAGES::Product::API::Object::LineItemProduct'}, {'string' => 'EUR'}, 'string']);
        return $Mockify->getMockObject();
    });


    return $Mockery;
}

sub _createExpectedLineItemsData {
    return [
          {
            'images' => [
                  {
                    'url' => 'http://image.baseurl.test.org/web/path/hotdeal',
                    'classifier' => 'HotDeal'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/large',
                    'classifier' => 'Large'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/medium',
                    'classifier' => 'Medium'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/small',
                    'classifier' => 'Small'
                  },{
                    'url' => 'http://image.baseurl.test.org/web/path/thumbnail',
                    'classifier' => 'Thumbnail'
                  }
                ],
            'sku' => 'abc_12345346',
            'quantity' => 2,
            'name' => 'Product 1',
            'lineItemPrice' => '32.99',
            'productId' => '245345-32534-324234',
            'lineItemId' => '123-35-56562-324352',
            'singleItemPrice' => '32.99',
            'essentialFeatures' => 'test product',
            'taxClass' => {
                'taxClassId' => '123-456-7890',
                'name' => 'TaxClassName',
                'percentage' => 19
            },
            'deliveryWeight' => 9.78,
            'links' => [
                 {
                   'rel' => 'product',
                   'href' => 'http://baseurl.test.org/api/products/245345-32534-324234',
                 },
                 {
                    'rel' => 'taxClass',
                    'href' => 'http://baseurl.test.org/api/tax-classes/123-456-7890',
                 }
               ]
          }
        ];
}

sub _createExpectedLineItemsData_DownloadableProduct {
    my $self = shift;
    my $Expected = $self->_createExpectedLineItemsData();
    @{$Expected}[0]->{'deliveryWeight'} = undef;
    return $Expected;
}


__PACKAGE__->runScript();