#========================================================================================
# package      DE_EPAGES::Packlink::t::Shop_si
# base         base DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for Packlink in UI/Shop
#========================================================================================
package DE_EPAGES::Packlink::t::Shop_si;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall );
use Test::More;
use Readonly;

use strict;

use constant SHOP_ID        => 'ShopID';
use constant WEBURLADMINSSL => 'WebUrlAdminSSL';

#========================================================================================
sub test {
    my $self = shift;
    $self->test_ViewPacklinkAuthorization_invalid_PacklinkParameters_missing_PacklinkID();
    $self->test_ViewPacklinkAuthorization_invalid_MerchantStatus_missing_PacklinkProviderMethod();
    $self->test_ViewPacklinkConsecution_ViewAppDetails();
    $self->test_ViewPacklinkConsecution_ViewShippingList();
    $self->test_GrantPacklinkAuthorization();
    done_testing();
}

#========================================================================================
sub test_ViewPacklinkAuthorization_invalid_PacklinkParameters_missing_PacklinkID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PacklinkID                  = undef;
    my $AppId                       = '12345';
    my $ValidShippingProviderMethod = '1';
    my $Shop               = $self->_createShop($ValidShippingProviderMethod);
    my $MockedUIShop       = $self->_createUIShop($Shop);
    my $MockedServlet      = $self->_createServlet($Shop, $PacklinkID, $AppId);

    $MockedUIShop->ViewPacklinkAuthorization($MockedServlet);

    is( $MockedServlet->vars('InvalidPacklinkParameters'), '1', "$SubTestName - Invalid Packlink Parameters" );
    is( $MockedServlet->vars('PacklinkConnectionValidated'), '0', "$SubTestName - Invalid Packlink Connection Validated" );
}

#========================================================================================
sub test_ViewPacklinkAuthorization_invalid_MerchantStatus_missing_PacklinkProviderMethod {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PacklinkID                  = '12345';
    my $AppId                       = '12345';
    my $ValidShippingProviderMethod = '0';
    my $Shop               = $self->_createShop($ValidShippingProviderMethod);
    my $MockedUIShop       = $self->_createUIShop($Shop);
    my $MockedServlet      = $self->_createServlet($Shop, $PacklinkID, $AppId);

    $MockedUIShop->ViewPacklinkAuthorization($MockedServlet);

    is( $MockedServlet->vars('NoPacklinkProviderMethod'), '1', "$SubTestName - No Packlink Provider Method" );
    is( $MockedServlet->vars('PacklinkConnectionValidated'), '0', "$SubTestName - Invalid Packlink Connection Validated" );
}

#========================================================================================
sub test_ViewPacklinkConsecution_ViewAppDetails {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PacklinkID                  = '12345';
    my $AppId                       = '12345';
    my $ValidShippingProviderMethod = '1';
    my $Shop               = $self->_createShop($ValidShippingProviderMethod);
    my $MockedUIShop       = $self->_createUIShop($Shop);
    my $MockedServlet      = $self->_createServlet($Shop, $PacklinkID, $AppId);

    $MockedUIShop->ViewPacklinkConsecution($MockedServlet);

    ok($MockedServlet->vars('cancelButtonViewURL') =~ /^[\?ViewAction=MBO-ViewAppDetails][.]*/i,"$SubTestName - URL contains MBO-ViewAppDetails");
}

#========================================================================================
sub test_ViewPacklinkConsecution_ViewShippingList {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PacklinkID                  = '12345';
    my $AppId                       = undef;
    my $ValidShippingProviderMethod = '1';
    my $Shop               = $self->_createShop($ValidShippingProviderMethod);
    my $MockedUIShop       = $self->_createUIShop($Shop);
    my $MockedServlet      = $self->_createServlet($Shop, $PacklinkID, $AppId);

    $MockedUIShop->ViewPacklinkConsecution($MockedServlet);

    ok($MockedServlet->vars('cancelButtonViewURL') =~ /^[\?ViewAction=MBO-ViewShippingList]/i,"$SubTestName - URL contains MBO-ViewShippingList");
}

#========================================================================================
sub test_GrantPacklinkAuthorization {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $PacklinkID                  = '12345';
    my $AppId                       = undef;
    my $ValidShippingProviderMethod = '1';
    my $Shop               = $self->_createShop($ValidShippingProviderMethod);
    my $MockedUIShop       = $self->_createUIShop($Shop);
    my $MockedServlet      = $self->_createServlet($Shop, $PacklinkID, $AppId);

    $MockedUIShop->GrantPacklinkAuthorization($MockedServlet);

    my $aParametersFromFirstCall = GetParametersFromMockifyCall($MockedUIShop , '_Redirect');
    ok(@$aParametersFromFirstCall[1] =~ /^WebUrlAdminSSL[.]*/i,"$SubTestName - URL contains MBO-ViewShippingList");
}

#========================================================================================
sub _createUIShop {
    my $self = shift;
    my ( $Shop ) = @_;

    my $MockedUIShop = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Packlink::UI::Shop');

    my $aParameterList = [ 'object', 'string' ];
    $MockedUIShop->addMethodSpyWithParameterCheck('_Redirect', $aParameterList);
    $MockedUIShop->addMockWithReturnValue('_createTools', $self->_createTools());

    my $Feature = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Feature', {});
    $MockedUIShop->addMockWithReturnValueAndParameterCheck('_getFeature', $Feature->getMockObject(), ['string']);

    my $FeatureCurrentValue = 3;
    $MockedUIShop->addMockWithReturnValueAndParameterCheck('_featureCurrentValue', $FeatureCurrentValue, ['object', 'object']);

    my $FeatureMaxValue = 4;
    $MockedUIShop->addMockWithReturnValueAndParameterCheck('_featureMaxValue', $FeatureMaxValue, ['object', 'object']);

    return  $MockedUIShop->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Shop, $PacklinkID, $AppId) = @_;

    my $MockedServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);

    $MockedServlet->addVars(
        {
            'InvalidPacklinkParameters'     => undef,
            'PacklinkConnectionValidated'   => undef,
            'cancelButtonViewURL'           => undef,
            'NoPacklinkProviderMethod'      => undef,
            'PacklinkID'                    => $PacklinkID,
            'appID'                         => $AppId,
        }
    );

    $MockedServlet->addParams( {
        'PacklinkID'  => $PacklinkID,
        'appID'       => $AppId,
    });

    my $aLogParameterList = [ 'string' ];
    $MockedServlet->addMockWithReturnValueAndParameterCheck('redirect', '1', $aLogParameterList);

    return $MockedServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my ($ValidShippingProviderMethod) = @_;

    my $hAttributes = {
        'ID' => SHOP_ID,
        'WebUrlAdminSSL' => WEBURLADMINSSL,
        'ShopTypeRef' => $self->_createShopTypeRef()
    };

    my $ShippingProviderMethod = $self->_getShippingProviderMethodPacklink($ValidShippingProviderMethod);

    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockedShop->set('Site' => $MockedShop);
    $MockedShop->set('Shop' => $MockedShop);

    $MockedShop->mock('folder', sub {
            shift;
            my ($FolderName) = @_;
            if($FolderName eq 'ShippingProviderMethods') {
                return $ShippingProviderMethod;
            }
    });

   return $MockedShop;
}

#----------------------------------------------------------------------------------------
sub _createTools {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Packlink::API::Tools');

    $Mockify->addMockWithReturnValueAndParameterCheck('IsPacklinkAppAvailable', 1, ['object']);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _getShippingProviderMethodPacklink {
    my $self = shift;
    my ($ValidShippingProviderMethod) = @_;

    my $MockedShippingProviderMethodPacklink = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Packlink::API::Object::ShippingProviderMethodPacklink');

    my $aLogParameterList = [ 'string' ];
    $MockedShippingProviderMethodPacklink->addMockWithReturnValueAndParameterCheck('existsChild',
            '1', $aLogParameterList);

    $aLogParameterList = [ 'string' ];
    $MockedShippingProviderMethodPacklink->addMockWithReturnValueAndParameterCheck('child',
            $self->_createShippingMethodPacklink($ValidShippingProviderMethod), $aLogParameterList);

    return $MockedShippingProviderMethodPacklink->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShippingMethodPacklink {
    my $self = shift;
    my ($ValidShippingProviderMethod) = @_;

    my $MockedShippingMethodPacklink = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Packlink::API::ShippingMethodPacklink');

    my $FinalMockedShippingMethodPacklink = undef;
    if( $ValidShippingProviderMethod ){
         $FinalMockedShippingMethodPacklink = $MockedShippingMethodPacklink->getMockObject();
    }

    return $FinalMockedShippingMethodPacklink;
}

#----------------------------------------------------------------------------------------
sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = { 'Alias' => 'Demo' };
    my $ShopTypeRef =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes );

    return $ShopTypeRef;
}

__PACKAGE__->runScript();
1;