#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::UI_PaymentMethodPayPalPlus
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::PaymentMethodPayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::UI_PaymentMethodPayPalPlus;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use Test::Exception;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->test_ViewSettings();
    $self->test_ViewSettings_NoSslCertificate();
    $self->test_ViewSettings_DefinedMerchantClientId();
    $self->test_ViewSettings_Error();
    $self->test_SaveSettings();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System);
    my $Servlet = $self->_build_Servlet_UndefinedMerchantClientId();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if ViewSettings does not die");

    my $aParamsFromCall = GetParametersFromMockifyCall($Servlet, 'vars');
    my $WasCalled = WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileId');
    is_deeply($aParamsFromCall, ['OnboardingURL', 'onboardingURL'], "$SubTestName - tests if onboarding Url is right");
    ok($WasCalled, "$SubTestName - method '_createExperienceProfileId' was called");
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_NoSslCertificate {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '0';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System);
    my $Servlet = $self->_build_Servlet_UndefinedMerchantClientId();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if ViewSettings does not die");

    my $aParamsFromCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    my $aParamsFromCall1 = GetParametersFromMockifyCall($Servlet, 'vars', 1);
    my $WasCalled = WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileId');
    is_deeply($aParamsFromCall0, ['NoValidSSLCertificateError', '1'], "$SubTestName - tests if error was set to Servlet");
    is_deeply($aParamsFromCall1, ['OnboardingURL', 'onboardingURL'], "$SubTestName - tests if onboarding Url is right");
    ok($WasCalled, "$SubTestName - method '_createExperienceProfileId' was called");
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_DefinedMerchantClientId {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus($System);
    my $Servlet = $self->_build_Servlet_DefinedMerchantClientId();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if ViewSettings does not die");

    my $aParamsFromCall = GetParametersFromMockifyCall($Servlet, 'vars');
    my $WasCalled = WasCalled($UI_PaymentMethodPayPalPlus, '_createExperienceProfileId');
    is_deeply($aParamsFromCall, ['OnboardingURL', 'onboardingURL'], "$SubTestName - tests if onboarding Url is right");
    is($WasCalled, 1, "$SubTestName - method '_createExperienceProfileId' was called");
}
#----------------------------------------------------------------------------------------
sub test_ViewSettings_Error {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $HasSSLCertificate = '1';
    my $System = $self->_createSystem($HasSSLCertificate);
    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus_BuildOnboardingURLError($System);
    my $Servlet = $self->_build_Servlet_UndefinedMerchantClientId();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->ViewSettings($Servlet) }, "$SubTestName - tests if ViewSettings does not die");

    my $aParamsFromCall0 = GetParametersFromMockifyCall($Servlet, 'vars', 0);
    my $aParamsFromCall1 = GetParametersFromMockifyCall($Servlet, 'vars', 1);
    is_deeply($aParamsFromCall0, ['OnboardingError', 1], "$SubTestName - tests if there was an Error");
    is_deeply($aParamsFromCall1, ['OnboardingErrorCode', 'myError'], "$SubTestName - tests if Errorcode is right");
}
#----------------------------------------------------------------------------------------
sub test_SaveSettings {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $UI_PaymentMethodPayPalPlus = $self->_build_UI_PaymentMethodPaypalPlus();
    my $Servlet = $self->_build_Servlet_UndefinedMerchantClientId();

    lives_ok(sub { $UI_PaymentMethodPayPalPlus->SaveSettings($Servlet) }, "$SubTestName - tests if SaveSettings does not die");
}
#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($MerchantClientId) = @_;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::Servlet::Permission',
         []
    );
    $Mockify->addMockWithReturnValue('object', $self->_createPaymentMethodPayPalPlusWithAttributeValues($MerchantClientId));
    $Mockify->addMockWithReturnValue('form', $self->_createForm());
    $Mockify->addMockWithReturnValueAndParameterCheck('vars', undef, ['string', 'string']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::Presentation::API::Form',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'form',
        undef,
        [
            {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
            {'string' => 'PayPalPlusSettings'}
        ]
    );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'ID' => '23456',
        'OrderProcess' => 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAttributes );
    return $Mockify->getMockObject();;
}
#----------------------------------------------------------------------------------------
sub _createSystem {
    my $self = shift;
    my ($HasSSLCertificate) = @_;

    my $hAttributes = {
        'HasSSLCertificate' => $HasSSLCertificate
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createClass {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::Object::API::Object::Class', {} );
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlusWithAttributeValues {
    my $self = shift;
    my ( $MerchantClientId ) = @_;

    my $hAttributes = {
        'ID' => '12345',
        'ExperienceProfileId' => undef,
        'Site' => $self->_createShop(),
        'ClientId' => 'ClientId_PaymentMethod',
        'ClientSecret' => 'ClientSecret_PaymentMethod',
        'MerchantClientId' => $MerchantClientId,
        'Shop' => $self->_createShop(),
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );

    my $Class = $self->_createClass();
    $Mockify->mock('class', sub {return $Class;});

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _build_Servlet_UndefinedMerchantClientId {
    my $self = shift;

    my $MerchantClientId = undef;

    return $self->_createServlet($MerchantClientId);
}

#----------------------------------------------------------------------------------------
sub _build_Servlet_DefinedMerchantClientId {
    my $self = shift;

    my $MerchantClientId = 'MerchantClientId';

    return $self->_createServlet($MerchantClientId);
}
#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus {
    my $self = shift;
    my ($System) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createMerchantOnboardingURLBuilder' => $self->_createMerchantOnboardingURLBuilder(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService(),
        '_LoadRootObject' => $System
    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}
#----------------------------------------------------------------------------------------
sub _build_UI_PaymentMethodPaypalPlus_BuildOnboardingURLError {
    my $self = shift;
    my ($System) = @_;

    my $hReturnValues = {
        '_createExperienceProfileIdService' => $self->_createExperienceProfileIdService(),
        'attributeValues' => {'Values' => {'ClientSecret' => 'ClientSecret', 'ClientId' => 'ClientId'}},
        '_createMerchantOnboardingURLBuilder' => $self->_createMerchantOnboardingURLBuilder_Error(),
        '_createGetMerchantStatusService' => $self->_createGetMerchantStatusService(),
        '_LoadRootObject' => $System
    };
    return $self->_create_UI_PaymentMethodPaypalPlus( $hReturnValues );
}
#----------------------------------------------------------------------------------------
sub _create_UI_PaymentMethodPaypalPlus {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::UI::PaymentMethodPayPalPlus',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('Save', undef, [{'object' => 'DE_EPAGES::Presentation::Servlet::Permission'}]);
    $Mockify->addMockWithReturnValueAndParameterCheck('_createExperienceProfileIdService', $hReturnValues->{'_createExperienceProfileIdService'} , [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]);
    $Mockify->addMethodSpy('_createExperienceProfileId');
    $Mockify->addMockWithReturnValue('_createGetMerchantStatusService', $hReturnValues->{'_createGetMerchantStatusService'});
    $Mockify->addMockWithReturnValueAndParameterCheck('_BuildShopUrl', 'ReturnURL', [
        {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
        {'hashref' => {'ViewAction' => 'MBO-ViewSettings'}},
        'hashref',
        {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'}
    ]);
    $Mockify->addMockWithReturnValue('_LoadRootObject', $hReturnValues->{'_LoadRootObject'});
    $Mockify->addMockWithReturnValueAndParameterCheck('_encode_base64', 'EncodedReturnURL', [
        {'string' => 'ReturnURL'},
        {'string' => ''}
    ]);
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'attributeValues',
        $hReturnValues->{'attributeValues'},
        [
            {'object' => 'DE_EPAGES::Presentation::Servlet::Permission'},
            {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
            {'object' => 'DE_EPAGES::Object::API::Object::Class'}
        ]
    );
    $Mockify->addMockWithReturnValue('_createMerchantOnboardingURLBuilder', $hReturnValues->{'_createMerchantOnboardingURLBuilder'});

    my $PaymentMethodPaypalPlus = $Mockify->getMockObject();
    return $PaymentMethodPaypalPlus;
}
#----------------------------------------------------------------------------------------
sub _createExperienceProfileIdService {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::ExperienceProfileId::Service',
         []
    );
    $Mockify->addMockWithReturnValue('doRequest', 1);
    $Mockify->addMockWithReturnValue('persistResponse', 1);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createGetMerchantStatusService {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::PayPalPlus::API::Webservice::GetMerchantStatus::Service',
         []
    );
    $Mockify->addMockWithReturnValueAndParameterCheck('doRequest', 1, [{'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'}]);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createMerchantOnboardingURLBuilder {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('getOnboardingURL', 'onboardingURL');
    $Mockify->addMockWithReturnValueAndParameterCheck('buildOnboardingURL', 1, [
        {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
        {'string'=>'EncodedReturnURL'}
    ]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createMerchantOnboardingURLBuilder_Error {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalPlus::API::MerchantOnboardingURLBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('getError', 'myError');
    $Mockify->addMockWithReturnValueAndParameterCheck('buildOnboardingURL', 0, [
        {'object' => 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus'},
        {'string'=>'EncodedReturnURL'}
    ]);
    return $Mockify->getMockObject();
}
__PACKAGE__->runScript();
