#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneShopStatus
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::SageOne::UI::JSON
#========================================================================================
package DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneShopStatus;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyServlet;
use Test::More;

use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_SageOneShopStatus_Shop_Disconnected();
    $self->test_SageOneShopStatus_Shop_Connected();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_SageOneShopStatus_Shop_Disconnected {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_Shop_Disconnected');
    my $Servlet = $self->memory()->get('Servlet');

    $SUT->SageOneShopStatus($Servlet);

    is(GetCallCount($SUT, 'writeContentOutput'), 1, "$SubTestName - checks if writeContentOutput was called only once");
    my $aReturnParams = GetParametersFromMockifyCall($SUT, 'writeContentOutput');
    is_deeply($aReturnParams->[0]->{'ShopStatus'}, 'disconnected', "$SubTestName - checks if the shopstatus is disconnected");
}
#----------------------------------------------------------------------------------------
sub test_SageOneShopStatus_Shop_Connected {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->memory()->get('SUT_Shop_Connected');
    my $Servlet = $self->memory()->get('Servlet');

    $SUT->SageOneShopStatus($Servlet);

    is(GetCallCount($SUT, 'writeContentOutput'), 1, "$SubTestName - checks if writeContentOutput was called only once");
    my $aReturnParams = GetParametersFromMockifyCall($SUT, 'writeContentOutput');
    my $aExpectedParams = [ {
            'ShopStatus' => 'connected',
            'SynchronizationStatus' => 'busy',
            'LastSuccessfulSynchronization' => '12.08.1492 13:14'
          } ];

    is_deeply($aReturnParams, $aExpectedParams, "$SubTestName - checks if Shopstatus is conected and status is busy and the date is in german datetime format");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->_createShop();
    my $Servlet = $self->_createServlet($Shop);

    my $ShopService_Connected = $self->_createShopServiceConnected();
    my $ShopService_DisConnected = $self->_createShopServiceDisconnected();

    $self->memory()->add('Servlet', $Servlet);
    $self->memory()->add('SUT_Shop_Connected', $self->_createSUT($Servlet, $ShopService_Connected));
    $self->memory()->add('SUT_Shop_Disconnected', $self->_createSUT($Servlet, $ShopService_DisConnected));
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Servlet, $ShopService) = @_;

    my $aParameterList = ['Servlet' => $Servlet];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::UI::JSON',
                   $aParameterList
              );
    $Mockify->mock('_createShopService', $ShopService, [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
    $Mockify->mock('writeContentOutput', 'DummyResponse', ['hashref']);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopServiceDisconnected {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    my $ShopStatusDto = DE_EPAGES::SageOne::API::DTO::ShopStatus->new();
    $ShopStatusDto->connectionStatus('disconnected');

    $Mockify->mock('getShopStatus', $ShopStatusDto);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopServiceConnected {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    my $ShopStatusDto = DE_EPAGES::SageOne::API::DTO::ShopStatus->new();
    $ShopStatusDto->connectionStatus('connected');
    $ShopStatusDto->synchronizationStatus('busy');
    $ShopStatusDto->lastSuccessfulSynchronization('1492-08-12 13:14:15');

    $Mockify->mock('getShopStatus', $ShopStatusDto);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Shop) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $Shop );
    $Mockify->addVar('Locale', 'de_DE');

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my $Mockify =  $self->mockFactory()->buildShop({
            'Alias' => 'MyShopAlias',
            'RESTAPIURL' => 'http://external/base/url/',
            'ID' => 123,
            'Site' => $self->_createSite()
        });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;

    my $Mockify =  $self->mockFactory()->buildSite({
            'ID' => 123,
            'GUID' => 'MySiteGUID',
        });

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();