#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneSyncOrders
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for SageOne::UI::JSON.pm
#========================================================================================
package DE_EPAGES::SageOne::t::unit::UI_JSON_SageOneSyncOrders;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::Mockify qw ( GetCallCount );
use Test::More;
use DE_EPAGES::Test::Mock::MockifyServlet;
use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->test_SyncOrders();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_SyncOrders {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->memory()->get('Shop');
    my $Servlet = $self->_createServlet($Shop);
    my $SUT = $self->memory()->get('SUT');

    $SUT->SageOneSyncOrders($Servlet);

    my $ShopService = $self->memory()->get('ShopService');
    is(GetCallCount($ShopService, 'syncShop'), 1, "$SubTestName - tests if syncShop was called");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $Shop = $self->_createShop();

    $self->memory()->add('ShopService', $self->_createShopService());

    my $SUT = $self->_createSUT();
    $self->memory()->add('SUT', $SUT);
    $self->memory()->add('Shop', $Shop);
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::UI::JSON',
                   $aParameterList
              );
    $Mockify->mock(
        '_createShopService',
        $self->memory()->get('ShopService'),
        [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]
    );
    $Mockify->mock( 'writeContentOutput', undef, [{'hashref' => {}}] );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShopService {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->mock('syncShop', undef);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;
    my $Attributes = {
        'ID' => 123,
        'Site' => $self->_createSite(),
    };
    return $self->mockFactory()->buildShop($Attributes)->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createSite {
    my $self = shift;
    my $Attributes = {
        'GUID'=>'MySiteGUID',
    };
    return $self->mockFactory()->buildSite($Attributes)->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Shop) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyServlet->new( $Shop );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();