#========================================================================================
# package      DE_EPAGES::SageOne::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::SageOne::t::unit::UI_Shop;
use base DE_EPAGES::Test::API::UnitTest::Test;
use Test::More;
use Test::Exception;
use DE_EPAGES::Test::Mock::Mockify qw (WasCalled);
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::SageOne::API::DTO::AuthUrl;
use DE_EPAGES::SageOne::API::DTO::ShopStatus;
use DE_EPAGES::SageOne::API::Constants qw (
    SAGE_SHOP_STATUS_CONNECTED
    SAGE_SHOP_STATUS_DISCONNECTED
    SAGE_SYNC_STATUS_IDLE
);
use strict;

sub test {
    my $self = shift;
    $self->test_ViewSettings_disconnected_SuccessfulOnBoardingUrl();
    $self->test_ViewSettings_disconnected_ErrorOnBoardingUrl();
    $self->test_ViewSettings_connected();

    $self->test_sageOneDisconnect();
    done_testing();
}

sub test_ViewSettings_disconnected_SuccessfulOnBoardingUrl {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopStatus = $self->_createShopStatusDisconnectedDTO();
    my $AuthUrl = $self->_createAuthUrlDTO();
    my $ShopService = $self->_createShopService($ShopStatus, $AuthUrl);
    my $UI_Shop = $self->_createUIShopSuccessful($ShopService);

    my $Servlet = $self->_createServlet();
    $UI_Shop->ViewSettings($Servlet);

    is($Servlet->vars('ConnectUrl'), 'http://sageone/authurl', "$SubTestName - test if ConnectUrl is set correctly");
    is($Servlet->vars('WebserviceError'), undef, "$SubTestName - test if WebserviceError is set correctly");
    is($Servlet->vars('ConnectionStatus'), SAGE_SHOP_STATUS_DISCONNECTED, "$SubTestName - test if ConnectionStatus is set correctly");
}

sub test_ViewSettings_disconnected_ErrorOnBoardingUrl {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopService = $self->_createShopService_AuthUrl_ResponseError();
    my $UI_Shop = $self->_createUIShopSuccessful($ShopService);

    my $Servlet = $self->_createServlet();
    $UI_Shop->ViewSettings($Servlet);

    is($Servlet->vars('WebserviceError'), 1, "$SubTestName - test if WebserviceError is set correctly");
    is($Servlet->vars('ConnectUrl'), undef, "$SubTestName - test if ConnectUrl is set correctly");
}

sub test_ViewSettings_connected {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopStatus = $self->_createShopStatusConnectedDTO();
    my $ShopStatusServiceConnected = $self->_createShopService($ShopStatus);
    my $UI_Shop = $self->_createUIShopSuccessful($ShopStatusServiceConnected);

    my $Servlet = $self->_createServlet();
    $UI_Shop->ViewSettings($Servlet);

    is($Servlet->vars('ConnectUrl'), undef, "$SubTestName - test if ConnectUrl is set correctly");
    is($Servlet->vars('WebserviceError'), undef, "$SubTestName - test if WebserviceError is set correctly");

    is($Servlet->vars('ConnectionStatus'), SAGE_SHOP_STATUS_CONNECTED, "$SubTestName - test if ConnectionStatus is set correctly");
    is($Servlet->vars('SynchronizationStatus'), SAGE_SYNC_STATUS_IDLE, "$SubTestName - test if SynchronizationStatus is set correctly");
    is($Servlet->vars('LastSuccessfulSynchronization'), '2016-04-28T09:10:54', "$SubTestName - test if LastSuccessfulSynchronization is set correctly");
}

sub test_sageOneDisconnect {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->_createServlet();
    my $ShopService = $self->_createShopService();
    my $UI_Shop = $self->_createUIShopSuccessful($ShopService);

    $UI_Shop->SageOneDisconnect($Servlet);

    my $WasCalled = WasCalled($ShopService, 'disconnectShop');
    is($WasCalled, 1, "$SubTestName - test if disconnectShop was called");
}

sub _createServlet {
    my $self = shift;
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $self->_createShop() );
    return $MockifyServlet->getMockObject();
}

sub _createUIShopSuccessful {
    my $self = shift;
    my ($ShopService) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::SageOne::UI::Shop', [] );
    $Mockify->addMockWithConditionalReturn(
        '_BuildShopUrl',
        'http://epages6returnurl.com/', [
            {'object' => 'DE_EPAGES::Shop::API::Object::Shop'},
            {'hashref' => { 'ViewAction' => 'MBO-SageOneSettings' }},
            {'hashref' => {'Type' => 'admin', 'UseSSL' => 1, 'AddToken' => 1}},
            {'object' => 'DE_EPAGES::WebInterface::Servlet::BaseServlet'},
        ]
    );

    $Mockify->addMockWithReturnValueAndParameterCheck(
        '_createShopService',
        $ShopService,
        [{'object'=>'DE_EPAGES::Shop::API::Object::Shop'}]
    );
    return $Mockify->getMockObject();
}

sub _createShopService {
    my $self = shift;
    my ($ShopStatus, $AuthUrl) = @_;
    my $aParameterList = [$self->_createShop()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValue('getShopStatus', $ShopStatus);
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'getAuthUrl',
        $AuthUrl,
        [
            {'string' => 'http://epages6returnurl.com/'},
            {'string' => 'http://epagesj/base/url/rs'}
        ]);
    $Mockify->addMockWithReturnValue('disconnectShop', undef);
    return $Mockify->getMockObject();
}

sub _createShopService_AuthUrl_ResponseError {
    my $self = shift;

    my $aParameterList = [$self->_createShop()];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::SageOne::API::Webservice::ShopService',
                   $aParameterList
              );
    my $ShopStatus = $self->_createShopStatusDisconnectedDTO();
    $Mockify->addMockWithReturnValue('getShopStatus', $ShopStatus);
    $Mockify->addMock('getAuthUrl', sub {
        Error('ResponseError');
    });

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'GUID'=>'MyShopGUID',
        'RESTAPIURL' => 'http://epagesj/base/url/',
        'ProtocolAndServerSSL' => 'http://epagesj/base/url/',
        'ID' => '18847',
        'Site' => $self->_createSite()
    };
    my $MockifyEpagesObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $MockifyEpagesObject->getMockObject();
}

sub _createSite {
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', {
            'ID' => 123,
            'GUID'=>'MySiteGUID',
        }
    );
    return $Mockify->getMockObject();
}

sub _createShopStatusDisconnectedDTO {
    my $self = shift;
    my $ShopStatusDTO = DE_EPAGES::SageOne::API::DTO::ShopStatus->new();
    $ShopStatusDTO->shopId('DisconnectedShopId');
    $ShopStatusDTO->connectionStatus(SAGE_SHOP_STATUS_DISCONNECTED);
    $ShopStatusDTO->synchronizationStatus(SAGE_SYNC_STATUS_IDLE);
    $ShopStatusDTO->lastSuccessfulSynchronization('2016-04-28 09:10:54');
    return $ShopStatusDTO;
}

sub _createShopStatusConnectedDTO {
    my $self = shift;
    my $ShopStatusDTO = DE_EPAGES::SageOne::API::DTO::ShopStatus->new();
    $ShopStatusDTO->shopId('ConnectedShopId');
    $ShopStatusDTO->connectionStatus(SAGE_SHOP_STATUS_CONNECTED);
    $ShopStatusDTO->synchronizationStatus(SAGE_SYNC_STATUS_IDLE);
    $ShopStatusDTO->lastSuccessfulSynchronization('2016-04-28 09:10:54');
    return $ShopStatusDTO;
}

sub _createAuthUrlDTO {
    my $self = shift;
    my $AuthUrlDTO = DE_EPAGES::SageOne::API::DTO::AuthUrl->new();
    $AuthUrlDTO->authUrl('http://sageone/authurl');
    return $AuthUrlDTO;
}
__PACKAGE__->runScript();
