#========================================================================================
# package      DE_EPAGES::Shippo::t::unit::Tools
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::Shippo::API::Tools
#========================================================================================
package DE_EPAGES::Shippo::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Shippo::API::Tools;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Exception;
use Test::More;

use strict;

use constant APPSTORE_FEATURE => 'AppStore';

sub test {
    my $self = shift;

    $self->_test_IsAppStoreVisible_negative();
    $self->_test_IsAppStoreVisible_positive();
    $self->_test_IsShippoAppAvailable_AppStoreNotVisible_negative();
    $self->_test_IsShippoAppAvailable_GetShippoAppIdError_negative();
    $self->_test_IsShippoAppAvailable_GetShippoAppIdUndef_negative();
    $self->_test_IsShippoAppAvailable_positive();

    done_testing();
}

#---------------------------------------------------------------------------
#  Tests
#---------------------------------------------------------------------------
sub _test_IsAppStoreVisible_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_createTools();
    my $Shop = $self->_buildShop_AppStoreNotVisible();
    my $IsVisible = $MockedTools->IsAppStoreVisible($Shop);

    ok(!$IsVisible, "$SubTestName - AppStore is not visible.");

    return;
}

sub _test_IsAppStoreVisible_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_createTools();
    my $Shop = $self->_buildShop_AppStoreVisible();
    my $IsVisible = $MockedTools->IsAppStoreVisible($Shop);

    ok($IsVisible, "$SubTestName - AppStore is visible.");

    return;
}

sub _test_IsShippoAppAvailable_AppStoreNotVisible_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_createTools();
    my $Shop = $self->_buildShop_AppStoreNotVisible();
    my $IsAvailable = $MockedTools->IsShippoAppAvailable($Shop);

    ok(!$IsAvailable, "$SubTestName - App is not available.");

    return;
}

sub _test_IsShippoAppAvailable_GetShippoAppIdError_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_buildTools_GetShippoAppIdError();
    my $Shop = $self->_buildShop_AppStoreVisible();
    my $IsAvailable = $MockedTools->IsShippoAppAvailable($Shop);

    ok(!$IsAvailable, "$SubTestName - App is not available.");

    return;
}

sub _test_IsShippoAppAvailable_GetShippoAppIdUndef_negative {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_buildTools_GetShippoAppIdUndef();
    my $Shop = $self->_buildShop_AppStoreVisible();
    my $IsAvailable = $MockedTools->IsShippoAppAvailable($Shop);

    ok(!$IsAvailable, "$SubTestName - App is not available.");

    return;
}

sub _test_IsShippoAppAvailable_positive {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $MockedTools = $self->_buildTools_GetShippoAppId();
    my $Shop = $self->_buildShop_AppStoreVisible();
    my $IsAvailable = $MockedTools->IsShippoAppAvailable($Shop);

    ok($IsAvailable, "$SubTestName - App is available.");

    return;
}

#---------------------------------------------------------------------------
#  Helpers
#---------------------------------------------------------------------------
sub _buildShop_AppStoreNotVisible {
    my $self = shift;

    my $hReturnValues = {
        'featureIsVisible' => 0,
    };

    return $self->_createShop($hReturnValues);
}

sub _buildShop_AppStoreVisible {
    my $self = shift;

    my $hReturnValues = {
        'featureIsVisible' => 1,
    };

    return $self->_createShop($hReturnValues);
}

sub _buildTools_GetShippoAppIdError {
    my $self = shift;

    my $hReturnValues;

    return $self->_createTools($hReturnValues);

}

sub _buildTools_GetShippoAppIdUndef {
    my $self = shift;

    my $hReturnValues = {
        'getShippoAppId' => undef,
    };

    return $self->_createTools($hReturnValues);
}

sub _buildTools_GetShippoAppId {
    my $self = shift;

    my $hReturnValues = {
        'getShippoAppId' => '12345',
    };

    return $self->_createTools($hReturnValues);
}

sub _createFeature {
    my $self = shift;

    my $hParameters = {
        'Alias' => 'AppStore',
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', $hParameters);

    return $Mockify->getMockObject();
}

sub _createShippoAppStoreShippo {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shippo::API::AppStoreShippo');

    $Mockify->addMock('getShippoAppId', sub {
        shift;

        Error('Error') if !isValid($hReturnValues);

        return $hReturnValues->{'getShippoAppId'};
    });

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});

    $Mockify->addMockWithReturnValueAndParameterCheck('featureIsVisible', $hReturnValues->{'featureIsVisible'}, ['object']);

    return $Mockify->getMockObject();
}

sub _createTools {
    my $self = shift;
    my ($hReturnValues) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Shippo::API::Tools');

    $Mockify->addMockWithReturnValueAndParameterCheck('_getFeature', $self->_createFeature(), [{'string' => APPSTORE_FEATURE}]);
    $Mockify->addMockWithReturnValue('_createAppStoreShippo', $self->_createShippoAppStoreShippo($hReturnValues));

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
