#========================================================================================
# package      DE_EPAGES::Shop::t::RestApiIntegrationTest
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  Integration test for RestApi resources.
#               Checks if requests are matched by the correct resources.
#========================================================================================
package DE_EPAGES::Shop::t::RestApiIntegrationTest;
use base DE_EPAGES::Shop::API::TestImportXML;

use DE_EPAGES::WebService::API::ResourceRegistry;
use HTTP::Request;
use Test::Exception;
use Test::More;

use strict;



sub shopAlias {
    return 'TestShop';
}



sub importXMLPath {
    return 'ShopXMLs/Shop_for_RestApiIntegration_test.xml';
}



sub test {
    my $self = shift;

    $self->test_get_publicInfo();
    $self->test_post_publicInfo();

    $self->test_get_locales();

    $self->test_get_currencies();

    done_testing();
}



sub test_get_publicInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/public-info/?locale=de_DE', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Shop::API::Resource::PublicInfoResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
    is($aParams->[1], 'de_DE', "$SubTestName - test if locale was extracted correctly");
}



sub test_post_publicInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('POST', '/api/public-info/', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Shop::API::Resource::PublicInfoResource', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'post', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub test_get_locales {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/locales/', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Shop::API::Resource::Locales', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub test_get_currencies {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Shop = $self->shop();
    my $Request = $self->_createRequest('GET', '/api/currencies/', 'application/vnd.epages.v1+json');

    my $hRequestContext = {};
    lives_ok(sub {
        $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $Shop, $Request );
    }, "$SubTestName - test if route can be matched.");

    my $Action = $hRequestContext->{'Action'};
    is($Action->get('Package'), 'DE_EPAGES::Shop::API::Resource::Currencies', "$SubTestName - test if route was matched by correct resource");
    is($Action->get('FunctionName'), 'get', "$SubTestName - test if route was matched by correct function");

    my $aParams = $hRequestContext->{'Params'};
    isa_ok($aParams->[0], 'DE_EPAGES::Shop::API::Object::Shop', "$SubTestName - test if object was extracted correctly");
}



sub _createRequest {
    my $self = shift;
    my ($Method, $Route, $Accept, $Content) = @_;

    my $URL = 'http://www.example.com'.$Route;
    my $Request = HTTP::Request->new($Method => $URL);

    if(defined $Accept) {
        $Request->header( 'Accept' => $Accept );
    }
    if(defined $Content) {
        warn "set content: $Content \n";
        $Request->content( $Content );
    }

    return $Request;
}



__PACKAGE__->runScript();

1;