#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::PaymentMethodSpreedly
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for API::Object::PaymentMethodSpreedly
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::PaymentMethodSpreedly;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly;

use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use constant PAYMENT_TYPE_METHOD_IS_SET => 1;
use constant PAYMENT_TYPE_METHOD_IS_NOT_SET => 0;
use constant TOKEN_IS_RECOVERED => 1;
use constant TOKEN_IS_NOT_RECOVERED => 0;
use constant DEFAULT_PAYMENT_METHOD_ALIAS => 'PaymentMethodAlias';
use constant DEFAULT_PAYMENT_TYPE_NAME =>'Default Credit Card Name';
use constant ANOTHER_PAYMENT_TYPE_NAME => 'Another Credit Card Name';
use constant DEFAULT_PAYMENT_TYPE_ALIAS =>'PaymentTypeMethodAlias';
use constant ANOTHER_PAYMENT_TYPE_ALIAS =>'AnotherPaymentTypeMethodAlias';
use constant LANGUAGE_ID => 1;

use strict;


sub test {
    my $self = shift;

    $self->_test_usable_at_store_front();
    $self->_test_not_usable_at_store_front_no_payment_type_method();
    $self->_test_not_usable_at_store_front_no_token();

    $self->_test_createNewPaymentMethodName();
    $self->_test_setPaymentMethodNameLikePaymentTypeMethod();

    $self->_test_setInShopNameLikePaymentTypeMethod_without_type_method_parameter();
    $self->_test_setInShopNameLikePaymentTypeMethod_with_type_method_parameter();

    $self->_test_uploadPaymentTypeMethodLogo_without_type_method_parameter();
    $self->_test_uploadPaymentTypeMethodLogo_with_type_method_parameter();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_usable_at_store_front {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedPaymentMethod = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_SET, TOKEN_IS_RECOVERED);

    ok( $MockedPaymentMethod->usableAtStorefront(), "$SubTestName - PaymentMethodSpreedly is usable at store front." );

    return;
}

sub _test_not_usable_at_store_front_no_payment_type_method {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedPaymentMethod = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_RECOVERED);

    ok( !$MockedPaymentMethod->usableAtStorefront(), "$SubTestName - PaymentMethodSpreedly is not usable at store front. No PaymentTypeMethod" );

    return;
}

sub _test_not_usable_at_store_front_no_token {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedPaymentMethod = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_SET, TOKEN_IS_NOT_RECOVERED);

    ok( !$MockedPaymentMethod->usableAtStorefront(), "$SubTestName - PaymentMethodSpreedly is not usable at store front. No Token" );

    return;
}

sub _test_createNewPaymentMethodName {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aPaymentMethods = [
          $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED, 'Visa Card'),
          $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED, 'Master-Card'),
          $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED, 'Master-Card_1'),
    ];
    my $PaymentMethodsObject = $self->_getPaymentMethodsObject($aPaymentMethods);

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED);

    my $BaseName = 'Maestro Card';
    is($PaymentMethodSpreedly->createNewPaymentMethodName($PaymentMethodsObject, $BaseName), 'Maestro Card',
            "$SubTestName - CreateNewPaymentMethodName returns expected name when it does not already exists");

    $BaseName = 'Master-Card';
    is($PaymentMethodSpreedly->createNewPaymentMethodName($PaymentMethodsObject, $BaseName), 'Master-Card_2',
            "$SubTestName - CreateNewPaymentMethodName returns expected name with counter sufix when it already exists");
}

sub _test_setPaymentMethodNameLikePaymentTypeMethod {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED);
    my $aPaymentMethods = [
          $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED, 'Visa Card'),
          $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED, 'Master-Card'),
    ];
    my $PaymentMethodsObject = $self->_getPaymentMethodsObject($aPaymentMethods);
    $PaymentMethodSpreedly->set( { 'Parent' => $PaymentMethodsObject } );

    my $PaymentTypeMethodToMirror = $self->_getPaymentTypeMethod();

    $PaymentMethodSpreedly->setPaymentMethodNameLikePaymentTypeMethod($PaymentTypeMethodToMirror, LANGUAGE_ID);

    is($PaymentMethodSpreedly->get('Alias'), DEFAULT_PAYMENT_TYPE_NAME,
            "$SubTestName - setPaymentMethodNameLikePaymentTypeMethod function test");
}

sub _test_setInShopNameLikePaymentTypeMethod_without_type_method_parameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_SET, TOKEN_IS_NOT_RECOVERED);
    $PaymentMethodSpreedly->setInShopNameLikePaymentTypeMethod();

    is($PaymentMethodSpreedly->get('Name'), DEFAULT_PAYMENT_TYPE_NAME,
            "$SubTestName - setInShopNameLikePaymentTypeMethod call without type method parameter");
}

sub _test_setInShopNameLikePaymentTypeMethod_with_type_method_parameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_NOT_SET, TOKEN_IS_NOT_RECOVERED);
    my $PaymentTypeMethodToMirror = $self->_getPaymentTypeMethod(ANOTHER_PAYMENT_TYPE_NAME);
    $PaymentMethodSpreedly->setInShopNameLikePaymentTypeMethod($PaymentTypeMethodToMirror);

    is($PaymentMethodSpreedly->get('Name'), ANOTHER_PAYMENT_TYPE_NAME,
            "$SubTestName - setInShopNameLikePaymentTypeMethod call with type method parameter");
}

sub _test_uploadPaymentTypeMethodLogo_without_type_method_parameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_SET, TOKEN_IS_NOT_RECOVERED);
    $PaymentMethodSpreedly->uploadPaymentTypeMethodLogo();

    is($PaymentMethodSpreedly->get('Image'), DEFAULT_PAYMENT_TYPE_ALIAS . '.png',
            "$SubTestName - uploadPaymentTypeMethodLogo call without type method parameter");
}

sub _test_uploadPaymentTypeMethodLogo_with_type_method_parameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $PaymentMethodSpreedly = $self->_getPaymentMethodSpreedly(PAYMENT_TYPE_METHOD_IS_SET, TOKEN_IS_NOT_RECOVERED);
    my $PaymentTypeMethod = $self->_getPaymentTypeMethod(DEFAULT_PAYMENT_TYPE_NAME, ANOTHER_PAYMENT_TYPE_ALIAS);
    $PaymentMethodSpreedly->uploadPaymentTypeMethodLogo($PaymentTypeMethod);

    is($PaymentMethodSpreedly->get('Image'), ANOTHER_PAYMENT_TYPE_ALIAS . '.png',
            "$SubTestName - uploadPaymentTypeMethodLogo call with type method parameter");
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _getPaymentMethodSpreedly {
    my $self = shift;
    my ($isPaymentTypeMethodSet, $isTokenRecovered, $PaymentMethodName) = @_;

    my $hAttributes = {
        'Alias'             => $PaymentMethodName //= DEFAULT_PAYMENT_METHOD_ALIAS,
        'FilePath'          => 'Shops/Shop/ABCD/1234',
        'Image'             => undef,
        'PaymentTypeMethod' => undef,
        'Site'              => $self->_getShop(),
    };
    $hAttributes->{'PaymentTypeMethod'} = $self->_getPaymentTypeMethod() if ($isPaymentTypeMethodSet);

    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Spreedly::API::Object::PaymentMethodSpreedly', $hAttributes
    );

    $MockifyPaymentMethod->mock('getGatewayCredentials',
            sub {
                if ($isTokenRecovered) {
                    return { 'token' => 'testGatewayToken' };
                } else {
                    return { 'token' => '' };
                }
            }
    );

    $MockifyPaymentMethod->mock('_LogDebug', sub { return; });

    $MockifyPaymentMethod->mock('_Transaction',
            sub {
                my $self = shift;
                my ($Code) = @_;
                &$Code;
            }
    );

    $MockifyPaymentMethod->mock('_ExistsFile', sub { return 1; });

    $MockifyPaymentMethod->mock('_CopyFile', sub { return; });

    $MockifyPaymentMethod->mock('_DeleteFile', sub { return 1; });

    $MockifyPaymentMethod->mock('_LoadRootObject', sub { return $self->_getSystem(); });

    return $MockifyPaymentMethod->getMockObject;
}

#----------------------------------------------------------------------------------------
sub _getPaymentTypeMethod {
    my $self = shift;
    my ($PaymentTypeName, $PaymentTypeAlias) = @_;

    my $hAttributes = {
        'Alias' => $PaymentTypeAlias //= DEFAULT_PAYMENT_TYPE_ALIAS,
        'Name' => $PaymentTypeName //= DEFAULT_PAYMENT_TYPE_NAME,
    };

    my $MockedPaymentTypeMethod = DE_EPAGES::Test::Mock::EpagesObject->new(
            'DE_EPAGES::ExternalPayment::API::PaymentTypeMethods', $hAttributes
    );

    return $MockedPaymentTypeMethod;
}

#----------------------------------------------------------------------------------------
sub _getShop {
    my $self = shift;

    my $hAttributes = {
        'Languages' => [
            { 'LanguageID' => 1 },
            { 'LanguageID' => 2 },
        ]
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
}

#----------------------------------------------------------------------------------------
sub _getPaymentMethodsObject {
    my $self = shift;
    my ($aPaymentMethods) = @_;

    my $MockifyPaymentMethods = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object', {}
    );
    $MockifyPaymentMethods->addMockWithReturnValue('children', $aPaymentMethods);

    return $MockifyPaymentMethods->getMockObject();
}

sub _getSystem {
    my $self = shift;

    my $hAttributes = {
        'StoreName' => 'Store',
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Object', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

#----------------------------------------------------------------------------------------
# Need to redefine lower calls (SUPER::usableAtStorefront)
#----------------------------------------------------------------------------------------
no warnings 'redefine'; ## no critic qw(ProhibitNoWarnings)
sub DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront {
    return 1;
}


__PACKAGE__->runScript();

1;
