#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::Tools
# state        public
# description  Unit test for Tools.
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;
use strict;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Spreedly::API::Tools;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use Readonly;
use Test::Exception;
use Test::More;

use constant URL   => 'https://teamoceanbcn-sea.intern.epages.de/MBOStuff';
use constant TOKEN => 'ThisIsMyToken';

sub test {
    my $self = shift;
    $self->_test_convertNumberToSpreedlyFormat();
    $self->_test_createDereferredURL();
    $self->_test_createDereferredURL_WithoutURL();
    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub _test_convertNumberToSpreedlyFormat {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools = $self->_createTools();
    ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    is( $MockedTools->convertNumberToSpreedlyFormat(100),       '10000',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat(100.),      '10000',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat(100.0),     '10000',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat(100.00),    '10000',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat(100.000),   '10000',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat(-100),      '-10000', "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat('123'),     '12300',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat('123.'),    '12300',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat('123.0'),   '12300',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat('123.00'),  '12300',  "$SubTestName - Returned number is correct" );
    is( $MockedTools->convertNumberToSpreedlyFormat('123.000'), '12300',  "$SubTestName - Returned number is correct" );
    ## use critic (ValuesAndExpressions::ProhibitMagicNumbers)

    return;
}

sub _test_createDereferredURL {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools = $self->_createTools();
    my $DereferredURL = $MockedTools->createDereferredURL(URL);
    my $ExpectedDereferredURL = URL . '&DereferrerToken=' . TOKEN;

    is($DereferredURL, $ExpectedDereferredURL, "$SubTestName - Dereferred URL matched expected value.");

    return;
}

sub _test_createDereferredURL_WithoutURL {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools = $self->_createTools();
    my $DereferredURL = $MockedTools->createDereferredURL();

    ok(!isValid($DereferredURL), "$SubTestName - Method did not return anything");
    ok(!WasCalled($MockedTools, '_GenerateDereferrerToken'), "$SubTestName - _GenerateDereferrerToken was not called.");

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createTools {
    my $self    = shift;

    my $MockifyTools = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::API::Tools');
    $MockifyTools->addMockWithReturnValueAndParameterCheck('_GenerateDereferrerToken', TOKEN, ['string']);

    return $MockifyTools->getMockObject();
}

__PACKAGE__->runScript();

1;
