#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::UI_Shop
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for UI::Shop
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::UI_Shop;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Spreedly::UI::Shop;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use Test::More;

use constant TRANSACTION_PAYMENT_NAME_DEFAULT => 'Spreedly';
use constant TRANSACTION_PAYMENT_NAME => 'SpreedlyTest';
use constant SEARCH_TEMPLATE_NAME =>  'SpreedlyTestTransactions';

use strict;


sub test {
    my $self = shift;

    $self->_test_TransactionPaymentName_withoutServeletParameter();
    $self->_test_TransactionPaymentName_withServeletParameter();
    $self->_test_SearchTransactions();

    done_testing();
}


#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_TransactionPaymentName_withoutServeletParameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    is(DE_EPAGES::Spreedly::UI::Shop->TransactionPaymentName(), TRANSACTION_PAYMENT_NAME_DEFAULT, "$SubTestName - transaction payment name returned having NO servelet parameter");
}

sub _test_TransactionPaymentName_withServeletParameter {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $ServletMock = $self->_createServlet();
    is(DE_EPAGES::Spreedly::UI::Shop->TransactionPaymentName($ServletMock), TRANSACTION_PAYMENT_NAME, "$SubTestName - transaction payment name returned having servelet parameter");
}

sub _test_SearchTransactions {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIShop = $self->_createUIShop();
    my $ServletMock = $self->_createServlet();
    $MockedUIShop->SearchTransactions($ServletMock);
    my $SearchForm = @{ GetParametersFromMockifyCall( $MockedUIShop, 'ExecuteTransactionSearch' ) }[1];

    ok( WasCalled( $MockedUIShop, 'ExecuteTransactionSearch' ),   "$SubTestName - ExecuteTransactionSearch has been called." );

    is ($SearchForm->{'Search'}, SEARCH_TEMPLATE_NAME, "$SubTestName - Search form template name generated has the right name.")
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;

    my $MockedShop = $self->_createShop();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new( $MockedShop );
    $MockifyServlet->addMockWithReturnValue( 'form', $self->_createForm() );
    $MockifyServlet->addParam( 'ViewAction',  TRANSACTION_PAYMENT_NAME);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});

    return $MockedShop;
}

sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::WebInterface::API::Form');

    $MockedForm->mock(
        'form',
        sub {
            return {};
        }
    );

    return $MockedForm;
}

sub _createUIShop {
    my $self = shift;

    my $MockifyUIShop = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::UI::Shop');

    my $aExecuteTransactionSearchParams = [ {'object' => 'DE_EPAGES::Presentation::Servlet::BasePermission'}, 'hashref' ];
    $MockifyUIShop->addMockWithReturnValueAndParameterCheck('ExecuteTransactionSearch', ['string'], $aExecuteTransactionSearchParams);

    return $MockifyUIShop->getMockObject();
}


__PACKAGE__->runScript();

1;
