#========================================================================================
# package      DE_EPAGES::Spreedly::t::unit::UI_Stripe
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::Spreedly::t::unit::UI_Stripe;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Spreedly::UI::Stripe;
use DE_EPAGES::Test::Mock::Mockify qw ( GetParametersFromMockifyCall WasCalled );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant URL   => 'https://teamoceanbcn-sea.intern.epages.de/MBOStuff';
use constant AuthorizationCode => 'ac_8NPktx3GXN4UBeZGsTEBXPZsv5koO0h5';

sub test {
    my $self = shift;

    $self->_test_BacklinkStripeOnboarding_WithValidURLAndAuthorizationCode();
    $self->_test_BacklinkStripeOnboarding_WithoutURL();
    $self->_test_BacklinkStripeOnboarding_WithoutAuthorizationCode();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub _test_BacklinkStripeOnboarding_WithValidURLAndAuthorizationCode {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIStripe = $self->_createUIStripe();
    my $MockedServlet    = $self->_buildServlet_WithValidURLAndAuthorizationCode();
    $MockedUIStripe->BacklinkStripeOnboarding($MockedServlet);

    my $aParameters = GetParametersFromMockifyCall( $MockedUIStripe, '_redirectToURL' );
    is_deeply( $aParameters, [ URL . '&code=' . AuthorizationCode ], "$SubTestName - _redirectToURL was called with the right parameters." );

    return;
}

sub _test_BacklinkStripeOnboarding_WithoutURL {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIStripe = $self->_createUIStripe();
    my $MockedServlet    = $self->_buildServlet_WithoutURL();
    $MockedUIStripe->BacklinkStripeOnboarding($MockedServlet);

    ok(!WasCalled( $MockedUIStripe, '_redirectToURL' ), "$SubTestName - _redirectToURL was not called.");

    return;
}

sub _test_BacklinkStripeOnboarding_WithoutAuthorizationCode {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUIStripe = $self->_createUIStripe();
    my $MockedServlet    = $self->_buildServlet_WithoutAuthorizationCode();
    $MockedUIStripe->BacklinkStripeOnboarding($MockedServlet);

    ok(!WasCalled( $MockedUIStripe, '_redirectToURL' ), "$SubTestName - _redirectToURL was not called.");

    return;
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _buildServlet_WithValidURLAndAuthorizationCode {
    my $self = shift;

    my $hVars = {
        'state' => URL,
        'code'  => AuthorizationCode,
    };

    return $self->_createServlet($hVars);
}

sub _buildServlet_WithoutURL {
    my $self = shift;

    my $hVars = {
        'code'  => AuthorizationCode,
    };

    return $self->_createServlet($hVars);
}

sub _buildServlet_WithoutAuthorizationCode {
    my $self = shift;

    my $hVars = {
        'state' => URL,
    };

    return $self->_createServlet($hVars);
}

sub _createForm {
    my $self = shift;
    my ($hVars) = @_;

    my $Form = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Presentation::API::Form');
    $Form->mock('value', sub {
        shift;
        my ($Value) = @_;

        return ($Value eq 'state') ? $hVars->{'state'} : $hVars->{'code'};
    });

    return $Form;
}

sub _createServlet {
    my $self = shift;
    my ($hVars) = @_;

    my $MockifyProvider = DE_EPAGES::Test::Mock::MockifyEpagesObject->new( 'DE_EPAGES::ShopConfiguration::API::Object::Provider', {} );
    my $Provider       = $MockifyProvider->getMockObject();
    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Provider);

    $MockifyServlet->addMockWithReturnValue('form', $self->_createForm($hVars));

    return $MockifyServlet->getMockObject();
}

sub _createUIStripe {
    my $self = shift;

    my $MockifyUIStripe = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Spreedly::UI::Stripe');

    my $aParams = [ 'string' ];
    $MockifyUIStripe->addMockWithReturnValueAndParameterCheck( '_redirectToURL', undef, $aParams );

    return $MockifyUIStripe->getMockObject();
}

__PACKAGE__->runScript();
